# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .beta_tool_param import BetaToolParam
from .beta_local_environment_param import BetaLocalEnvironmentParam
from .beta_easy_input_message_param import BetaEasyInputMessageParam
from .beta_container_reference_param import BetaContainerReferenceParam
from .beta_response_output_message_param import BetaResponseOutputMessageParam
from .beta_response_reasoning_item_param import BetaResponseReasoningItemParam
from .beta_response_custom_tool_call_param import BetaResponseCustomToolCallParam
from .beta_response_computer_tool_call_param import BetaResponseComputerToolCallParam
from .beta_response_function_tool_call_param import BetaResponseFunctionToolCallParam
from .beta_response_input_text_content_param import BetaResponseInputTextContentParam
from .beta_response_function_web_search_param import BetaResponseFunctionWebSearchParam
from .beta_response_input_image_content_param import BetaResponseInputImageContentParam
from .beta_response_compaction_item_param_param import BetaResponseCompactionItemParamParam
from .beta_response_file_search_tool_call_param import BetaResponseFileSearchToolCallParam
from .beta_response_custom_tool_call_output_param import BetaResponseCustomToolCallOutputParam
from .beta_response_code_interpreter_tool_call_param import BetaResponseCodeInterpreterToolCallParam
from .beta_response_input_message_content_list_param import BetaResponseInputMessageContentListParam
from .beta_response_tool_search_output_item_param_param import BetaResponseToolSearchOutputItemParamParam
from .beta_response_function_call_output_item_list_param import BetaResponseFunctionCallOutputItemListParam
from .beta_response_function_shell_call_output_content_param import BetaResponseFunctionShellCallOutputContentParam
from .beta_response_computer_tool_call_output_screenshot_param import BetaResponseComputerToolCallOutputScreenshotParam

__all__ = [
    "BetaResponseInputParam",
    "BetaResponseInputItemParam",
    "Message",
    "MessageAgent",
    "ComputerCallOutput",
    "ComputerCallOutputAcknowledgedSafetyCheck",
    "ComputerCallOutputAgent",
    "FunctionCallOutput",
    "FunctionCallOutputAgent",
    "FunctionCallOutputCaller",
    "FunctionCallOutputCallerDirect",
    "FunctionCallOutputCallerProgram",
    "AgentMessage",
    "AgentMessageContent",
    "AgentMessageContentEncryptedContent",
    "AgentMessageAgent",
    "MultiAgentCall",
    "MultiAgentCallAgent",
    "MultiAgentCallOutput",
    "MultiAgentCallOutputOutput",
    "MultiAgentCallOutputOutputAnnotation",
    "MultiAgentCallOutputOutputAnnotationFileCitation",
    "MultiAgentCallOutputOutputAnnotationURLCitation",
    "MultiAgentCallOutputOutputAnnotationContainerFileCitation",
    "MultiAgentCallOutputAgent",
    "ToolSearchCall",
    "ToolSearchCallAgent",
    "AdditionalTools",
    "AdditionalToolsAgent",
    "ImageGenerationCall",
    "ImageGenerationCallAgent",
    "LocalShellCall",
    "LocalShellCallAction",
    "LocalShellCallAgent",
    "LocalShellCallOutput",
    "LocalShellCallOutputAgent",
    "ShellCall",
    "ShellCallAction",
    "ShellCallAgent",
    "ShellCallCaller",
    "ShellCallCallerDirect",
    "ShellCallCallerProgram",
    "ShellCallEnvironment",
    "ShellCallOutput",
    "ShellCallOutputAgent",
    "ShellCallOutputCaller",
    "ShellCallOutputCallerDirect",
    "ShellCallOutputCallerProgram",
    "ApplyPatchCall",
    "ApplyPatchCallOperation",
    "ApplyPatchCallOperationCreateFile",
    "ApplyPatchCallOperationDeleteFile",
    "ApplyPatchCallOperationUpdateFile",
    "ApplyPatchCallAgent",
    "ApplyPatchCallCaller",
    "ApplyPatchCallCallerDirect",
    "ApplyPatchCallCallerProgram",
    "ApplyPatchCallOutput",
    "ApplyPatchCallOutputAgent",
    "ApplyPatchCallOutputCaller",
    "ApplyPatchCallOutputCallerDirect",
    "ApplyPatchCallOutputCallerProgram",
    "McpListTools",
    "McpListToolsTool",
    "McpListToolsAgent",
    "McpApprovalRequest",
    "McpApprovalRequestAgent",
    "McpApprovalResponse",
    "McpApprovalResponseAgent",
    "McpCall",
    "McpCallAgent",
    "CompactionTrigger",
    "CompactionTriggerAgent",
    "ItemReference",
    "ItemReferenceAgent",
    "Program",
    "ProgramAgent",
    "ProgramOutput",
    "ProgramOutputAgent",
]


class MessageAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class Message(TypedDict, total=False):
    """
    A message input to the model with a role indicating instruction following
    hierarchy. Instructions given with the `developer` or `system` role take
    precedence over instructions given with the `user` role.
    """

    content: Required[BetaResponseInputMessageContentListParam]
    """
    A list of one or many input items to the model, containing different content
    types.
    """

    role: Required[Literal["user", "system", "developer"]]
    """The role of the message input. One of `user`, `system`, or `developer`."""

    agent: Optional[MessageAgent]
    """The agent that produced this item."""

    status: Literal["in_progress", "completed", "incomplete"]
    """The status of item.

    One of `in_progress`, `completed`, or `incomplete`. Populated when items are
    returned via API.
    """

    type: Literal["message"]
    """The type of the message input. Always set to `message`."""


class ComputerCallOutputAcknowledgedSafetyCheck(TypedDict, total=False):
    """A pending safety check for the computer call."""

    id: Required[str]
    """The ID of the pending safety check."""

    code: Optional[str]
    """The type of the pending safety check."""

    message: Optional[str]
    """Details about the pending safety check."""


class ComputerCallOutputAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ComputerCallOutput(TypedDict, total=False):
    """The output of a computer tool call."""

    call_id: Required[str]
    """The ID of the computer tool call that produced the output."""

    output: Required[BetaResponseComputerToolCallOutputScreenshotParam]
    """A computer screenshot image used with the computer use tool."""

    type: Required[Literal["computer_call_output"]]
    """The type of the computer tool call output. Always `computer_call_output`."""

    id: Optional[str]
    """The ID of the computer tool call output."""

    acknowledged_safety_checks: Optional[Iterable[ComputerCallOutputAcknowledgedSafetyCheck]]
    """
    The safety checks reported by the API that have been acknowledged by the
    developer.
    """

    agent: Optional[ComputerCallOutputAgent]
    """The agent that produced this item."""

    status: Optional[Literal["in_progress", "completed", "incomplete"]]
    """The status of the message input.

    One of `in_progress`, `completed`, or `incomplete`. Populated when input items
    are returned via API.
    """


class FunctionCallOutputAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class FunctionCallOutputCallerDirect(TypedDict, total=False):
    type: Required[Literal["direct"]]
    """The caller type. Always `direct`."""


class FunctionCallOutputCallerProgram(TypedDict, total=False):
    caller_id: Required[str]
    """The call ID of the program item that produced this tool call."""

    type: Required[Literal["program"]]
    """The caller type. Always `program`."""


FunctionCallOutputCaller: TypeAlias = Union[FunctionCallOutputCallerDirect, FunctionCallOutputCallerProgram]


class FunctionCallOutput(TypedDict, total=False):
    """The output of a function tool call."""

    call_id: Required[str]
    """The unique ID of the function tool call generated by the model."""

    output: Required[Union[str, BetaResponseFunctionCallOutputItemListParam]]
    """Text, image, or file output of the function tool call."""

    type: Required[Literal["function_call_output"]]
    """The type of the function tool call output. Always `function_call_output`."""

    id: Optional[str]
    """The unique ID of the function tool call output.

    Populated when this item is returned via API.
    """

    agent: Optional[FunctionCallOutputAgent]
    """The agent that produced this item."""

    caller: Optional[FunctionCallOutputCaller]
    """The execution context that produced this tool call."""

    status: Optional[Literal["in_progress", "completed", "incomplete"]]
    """The status of the item.

    One of `in_progress`, `completed`, or `incomplete`. Populated when items are
    returned via API.
    """


class AgentMessageContentEncryptedContent(TypedDict, total=False):
    """
    Opaque encrypted content that Responses API decrypts inside trusted model execution.
    """

    encrypted_content: Required[str]
    """Opaque encrypted content."""

    type: Required[Literal["encrypted_content"]]
    """The type of the input item. Always `encrypted_content`."""


AgentMessageContent: TypeAlias = Union[
    BetaResponseInputTextContentParam, BetaResponseInputImageContentParam, AgentMessageContentEncryptedContent
]


class AgentMessageAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class AgentMessage(TypedDict, total=False):
    """A message routed between agents."""

    author: Required[str]
    """The sending agent identity."""

    content: Required[Iterable[AgentMessageContent]]
    """Plaintext, image, or encrypted content sent between agents."""

    recipient: Required[str]
    """The destination agent identity."""

    type: Required[Literal["agent_message"]]
    """The item type. Always `agent_message`."""

    id: Optional[str]
    """The unique ID of this agent message item."""

    agent: Optional[AgentMessageAgent]
    """The agent that produced this item."""


class MultiAgentCallAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class MultiAgentCall(TypedDict, total=False):
    action: Required[
        Literal["spawn_agent", "interrupt_agent", "list_agents", "send_message", "followup_task", "wait_agent"]
    ]
    """The multi-agent action that was executed."""

    arguments: Required[str]
    """The action arguments as a JSON string."""

    call_id: Required[str]
    """The unique ID linking this call to its output."""

    type: Required[Literal["multi_agent_call"]]
    """The item type. Always `multi_agent_call`."""

    id: Optional[str]
    """The unique ID of this multi-agent call."""

    agent: Optional[MultiAgentCallAgent]
    """The agent that produced this item."""


class MultiAgentCallOutputOutputAnnotationFileCitation(TypedDict, total=False):
    file_id: Required[str]
    """The ID of the file."""

    filename: Required[str]
    """The filename of the file cited."""

    index: Required[int]
    """The index of the file in the list of files."""

    type: Required[Literal["file_citation"]]
    """The citation type. Always `file_citation`."""


class MultiAgentCallOutputOutputAnnotationURLCitation(TypedDict, total=False):
    end_index: Required[int]
    """The index of the last character of the citation in the message."""

    start_index: Required[int]
    """The index of the first character of the citation in the message."""

    title: Required[str]
    """The title of the cited resource."""

    type: Required[Literal["url_citation"]]
    """The citation type. Always `url_citation`."""

    url: Required[str]
    """The URL of the cited resource."""


class MultiAgentCallOutputOutputAnnotationContainerFileCitation(TypedDict, total=False):
    container_id: Required[str]
    """The ID of the container."""

    end_index: Required[int]
    """The index of the last character of the citation in the message."""

    file_id: Required[str]
    """The ID of the container file."""

    filename: Required[str]
    """The filename of the container file cited."""

    start_index: Required[int]
    """The index of the first character of the citation in the message."""

    type: Required[Literal["container_file_citation"]]
    """The citation type. Always `container_file_citation`."""


MultiAgentCallOutputOutputAnnotation: TypeAlias = Union[
    MultiAgentCallOutputOutputAnnotationFileCitation,
    MultiAgentCallOutputOutputAnnotationURLCitation,
    MultiAgentCallOutputOutputAnnotationContainerFileCitation,
]


class MultiAgentCallOutputOutput(TypedDict, total=False):
    text: Required[str]
    """The text content."""

    type: Required[Literal["output_text"]]
    """The content type. Always `output_text`."""

    annotations: Iterable[MultiAgentCallOutputOutputAnnotation]
    """Citations associated with the text content."""


class MultiAgentCallOutputAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class MultiAgentCallOutput(TypedDict, total=False):
    action: Required[
        Literal["spawn_agent", "interrupt_agent", "list_agents", "send_message", "followup_task", "wait_agent"]
    ]
    """The multi-agent action that produced this result."""

    call_id: Required[str]
    """The unique ID of the multi-agent call."""

    output: Required[Iterable[MultiAgentCallOutputOutput]]
    """Text output returned by the multi-agent action."""

    type: Required[Literal["multi_agent_call_output"]]
    """The item type. Always `multi_agent_call_output`."""

    id: Optional[str]
    """The unique ID of this multi-agent call output."""

    agent: Optional[MultiAgentCallOutputAgent]
    """The agent that produced this item."""


class ToolSearchCallAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ToolSearchCall(TypedDict, total=False):
    arguments: Required[object]
    """The arguments supplied to the tool search call."""

    type: Required[Literal["tool_search_call"]]
    """The item type. Always `tool_search_call`."""

    id: Optional[str]
    """The unique ID of this tool search call."""

    agent: Optional[ToolSearchCallAgent]
    """The agent that produced this item."""

    call_id: Optional[str]
    """The unique ID of the tool search call generated by the model."""

    execution: Literal["server", "client"]
    """Whether tool search was executed by the server or by the client."""

    status: Optional[Literal["in_progress", "completed", "incomplete"]]
    """The status of the tool search call."""


class AdditionalToolsAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class AdditionalTools(TypedDict, total=False):
    role: Required[Literal["developer"]]
    """The role that provided the additional tools. Only `developer` is supported."""

    tools: Required[Iterable[BetaToolParam]]
    """A list of additional tools made available at this item."""

    type: Required[Literal["additional_tools"]]
    """The item type. Always `additional_tools`."""

    id: Optional[str]
    """The unique ID of this additional tools item."""

    agent: Optional[AdditionalToolsAgent]
    """The agent that produced this item."""


class ImageGenerationCallAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ImageGenerationCall(TypedDict, total=False):
    """An image generation request made by the model."""

    id: Required[str]
    """The unique ID of the image generation call."""

    result: Required[Optional[str]]
    """The generated image encoded in base64."""

    status: Required[Literal["in_progress", "completed", "generating", "failed"]]
    """The status of the image generation call."""

    type: Required[Literal["image_generation_call"]]
    """The type of the image generation call. Always `image_generation_call`."""

    agent: Optional[ImageGenerationCallAgent]
    """The agent that produced this item."""


class LocalShellCallAction(TypedDict, total=False):
    """Execute a shell command on the server."""

    command: Required[SequenceNotStr[str]]
    """The command to run."""

    env: Required[Dict[str, str]]
    """Environment variables to set for the command."""

    type: Required[Literal["exec"]]
    """The type of the local shell action. Always `exec`."""

    timeout_ms: Optional[int]
    """Optional timeout in milliseconds for the command."""

    user: Optional[str]
    """Optional user to run the command as."""

    working_directory: Optional[str]
    """Optional working directory to run the command in."""


class LocalShellCallAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class LocalShellCall(TypedDict, total=False):
    """A tool call to run a command on the local shell."""

    id: Required[str]
    """The unique ID of the local shell call."""

    action: Required[LocalShellCallAction]
    """Execute a shell command on the server."""

    call_id: Required[str]
    """The unique ID of the local shell tool call generated by the model."""

    status: Required[Literal["in_progress", "completed", "incomplete"]]
    """The status of the local shell call."""

    type: Required[Literal["local_shell_call"]]
    """The type of the local shell call. Always `local_shell_call`."""

    agent: Optional[LocalShellCallAgent]
    """The agent that produced this item."""


class LocalShellCallOutputAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class LocalShellCallOutput(TypedDict, total=False):
    """The output of a local shell tool call."""

    id: Required[str]
    """The unique ID of the local shell tool call generated by the model."""

    output: Required[str]
    """A JSON string of the output of the local shell tool call."""

    type: Required[Literal["local_shell_call_output"]]
    """The type of the local shell tool call output. Always `local_shell_call_output`."""

    agent: Optional[LocalShellCallOutputAgent]
    """The agent that produced this item."""

    status: Optional[Literal["in_progress", "completed", "incomplete"]]
    """The status of the item. One of `in_progress`, `completed`, or `incomplete`."""


class ShellCallAction(TypedDict, total=False):
    """The shell commands and limits that describe how to run the tool call."""

    commands: Required[SequenceNotStr[str]]
    """Ordered shell commands for the execution environment to run."""

    max_output_length: Optional[int]
    """
    Maximum number of UTF-8 characters to capture from combined stdout and stderr
    output.
    """

    timeout_ms: Optional[int]
    """Maximum wall-clock time in milliseconds to allow the shell commands to run."""


class ShellCallAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ShellCallCallerDirect(TypedDict, total=False):
    type: Required[Literal["direct"]]
    """The caller type. Always `direct`."""


class ShellCallCallerProgram(TypedDict, total=False):
    caller_id: Required[str]
    """The call ID of the program item that produced this tool call."""

    type: Required[Literal["program"]]
    """The caller type. Always `program`."""


ShellCallCaller: TypeAlias = Union[ShellCallCallerDirect, ShellCallCallerProgram]

ShellCallEnvironment: TypeAlias = Union[BetaLocalEnvironmentParam, BetaContainerReferenceParam]


class ShellCall(TypedDict, total=False):
    """A tool representing a request to execute one or more shell commands."""

    action: Required[ShellCallAction]
    """The shell commands and limits that describe how to run the tool call."""

    call_id: Required[str]
    """The unique ID of the shell tool call generated by the model."""

    type: Required[Literal["shell_call"]]
    """The type of the item. Always `shell_call`."""

    id: Optional[str]
    """The unique ID of the shell tool call.

    Populated when this item is returned via API.
    """

    agent: Optional[ShellCallAgent]
    """The agent that produced this item."""

    caller: Optional[ShellCallCaller]
    """The execution context that produced this tool call."""

    environment: Optional[ShellCallEnvironment]
    """The environment to execute the shell commands in."""

    status: Optional[Literal["in_progress", "completed", "incomplete"]]
    """The status of the shell call.

    One of `in_progress`, `completed`, or `incomplete`.
    """


class ShellCallOutputAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ShellCallOutputCallerDirect(TypedDict, total=False):
    type: Required[Literal["direct"]]
    """The caller type. Always `direct`."""


class ShellCallOutputCallerProgram(TypedDict, total=False):
    caller_id: Required[str]
    """The call ID of the program item that produced this tool call."""

    type: Required[Literal["program"]]
    """The caller type. Always `program`."""


ShellCallOutputCaller: TypeAlias = Union[ShellCallOutputCallerDirect, ShellCallOutputCallerProgram]


class ShellCallOutput(TypedDict, total=False):
    """The streamed output items emitted by a shell tool call."""

    call_id: Required[str]
    """The unique ID of the shell tool call generated by the model."""

    output: Required[Iterable[BetaResponseFunctionShellCallOutputContentParam]]
    """
    Captured chunks of stdout and stderr output, along with their associated
    outcomes.
    """

    type: Required[Literal["shell_call_output"]]
    """The type of the item. Always `shell_call_output`."""

    id: Optional[str]
    """The unique ID of the shell tool call output.

    Populated when this item is returned via API.
    """

    agent: Optional[ShellCallOutputAgent]
    """The agent that produced this item."""

    caller: Optional[ShellCallOutputCaller]
    """The execution context that produced this tool call."""

    max_output_length: Optional[int]
    """
    The maximum number of UTF-8 characters captured for this shell call's combined
    output.
    """

    status: Optional[Literal["in_progress", "completed", "incomplete"]]
    """The status of the shell call output."""


class ApplyPatchCallOperationCreateFile(TypedDict, total=False):
    """Instruction for creating a new file via the apply_patch tool."""

    diff: Required[str]
    """Unified diff content to apply when creating the file."""

    path: Required[str]
    """Path of the file to create relative to the workspace root."""

    type: Required[Literal["create_file"]]
    """The operation type. Always `create_file`."""


class ApplyPatchCallOperationDeleteFile(TypedDict, total=False):
    """Instruction for deleting an existing file via the apply_patch tool."""

    path: Required[str]
    """Path of the file to delete relative to the workspace root."""

    type: Required[Literal["delete_file"]]
    """The operation type. Always `delete_file`."""


class ApplyPatchCallOperationUpdateFile(TypedDict, total=False):
    """Instruction for updating an existing file via the apply_patch tool."""

    diff: Required[str]
    """Unified diff content to apply to the existing file."""

    path: Required[str]
    """Path of the file to update relative to the workspace root."""

    type: Required[Literal["update_file"]]
    """The operation type. Always `update_file`."""


ApplyPatchCallOperation: TypeAlias = Union[
    ApplyPatchCallOperationCreateFile, ApplyPatchCallOperationDeleteFile, ApplyPatchCallOperationUpdateFile
]


class ApplyPatchCallAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ApplyPatchCallCallerDirect(TypedDict, total=False):
    type: Required[Literal["direct"]]
    """The caller type. Always `direct`."""


class ApplyPatchCallCallerProgram(TypedDict, total=False):
    caller_id: Required[str]
    """The call ID of the program item that produced this tool call."""

    type: Required[Literal["program"]]
    """The caller type. Always `program`."""


ApplyPatchCallCaller: TypeAlias = Union[ApplyPatchCallCallerDirect, ApplyPatchCallCallerProgram]


class ApplyPatchCall(TypedDict, total=False):
    """
    A tool call representing a request to create, delete, or update files using diff patches.
    """

    call_id: Required[str]
    """The unique ID of the apply patch tool call generated by the model."""

    operation: Required[ApplyPatchCallOperation]
    """
    The specific create, delete, or update instruction for the apply_patch tool
    call.
    """

    status: Required[Literal["in_progress", "completed"]]
    """The status of the apply patch tool call. One of `in_progress` or `completed`."""

    type: Required[Literal["apply_patch_call"]]
    """The type of the item. Always `apply_patch_call`."""

    id: Optional[str]
    """The unique ID of the apply patch tool call.

    Populated when this item is returned via API.
    """

    agent: Optional[ApplyPatchCallAgent]
    """The agent that produced this item."""

    caller: Optional[ApplyPatchCallCaller]
    """The execution context that produced this tool call."""


class ApplyPatchCallOutputAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ApplyPatchCallOutputCallerDirect(TypedDict, total=False):
    type: Required[Literal["direct"]]
    """The caller type. Always `direct`."""


class ApplyPatchCallOutputCallerProgram(TypedDict, total=False):
    caller_id: Required[str]
    """The call ID of the program item that produced this tool call."""

    type: Required[Literal["program"]]
    """The caller type. Always `program`."""


ApplyPatchCallOutputCaller: TypeAlias = Union[ApplyPatchCallOutputCallerDirect, ApplyPatchCallOutputCallerProgram]


class ApplyPatchCallOutput(TypedDict, total=False):
    """The streamed output emitted by an apply patch tool call."""

    call_id: Required[str]
    """The unique ID of the apply patch tool call generated by the model."""

    status: Required[Literal["completed", "failed"]]
    """The status of the apply patch tool call output. One of `completed` or `failed`."""

    type: Required[Literal["apply_patch_call_output"]]
    """The type of the item. Always `apply_patch_call_output`."""

    id: Optional[str]
    """The unique ID of the apply patch tool call output.

    Populated when this item is returned via API.
    """

    agent: Optional[ApplyPatchCallOutputAgent]
    """The agent that produced this item."""

    caller: Optional[ApplyPatchCallOutputCaller]
    """The execution context that produced this tool call."""

    output: Optional[str]
    """
    Optional human-readable log text from the apply patch tool (e.g., patch results
    or errors).
    """


class McpListToolsTool(TypedDict, total=False):
    """A tool available on an MCP server."""

    input_schema: Required[object]
    """The JSON schema describing the tool's input."""

    name: Required[str]
    """The name of the tool."""

    annotations: Optional[object]
    """Additional annotations about the tool."""

    description: Optional[str]
    """The description of the tool."""


class McpListToolsAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class McpListTools(TypedDict, total=False):
    """A list of tools available on an MCP server."""

    id: Required[str]
    """The unique ID of the list."""

    server_label: Required[str]
    """The label of the MCP server."""

    tools: Required[Iterable[McpListToolsTool]]
    """The tools available on the server."""

    type: Required[Literal["mcp_list_tools"]]
    """The type of the item. Always `mcp_list_tools`."""

    agent: Optional[McpListToolsAgent]
    """The agent that produced this item."""

    error: Optional[str]
    """Error message if the server could not list tools."""


class McpApprovalRequestAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class McpApprovalRequest(TypedDict, total=False):
    """A request for human approval of a tool invocation."""

    id: Required[str]
    """The unique ID of the approval request."""

    arguments: Required[str]
    """A JSON string of arguments for the tool."""

    name: Required[str]
    """The name of the tool to run."""

    server_label: Required[str]
    """The label of the MCP server making the request."""

    type: Required[Literal["mcp_approval_request"]]
    """The type of the item. Always `mcp_approval_request`."""

    agent: Optional[McpApprovalRequestAgent]
    """The agent that produced this item."""


class McpApprovalResponseAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class McpApprovalResponse(TypedDict, total=False):
    """A response to an MCP approval request."""

    approval_request_id: Required[str]
    """The ID of the approval request being answered."""

    approve: Required[bool]
    """Whether the request was approved."""

    type: Required[Literal["mcp_approval_response"]]
    """The type of the item. Always `mcp_approval_response`."""

    id: Optional[str]
    """The unique ID of the approval response"""

    agent: Optional[McpApprovalResponseAgent]
    """The agent that produced this item."""

    reason: Optional[str]
    """Optional reason for the decision."""


class McpCallAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class McpCall(TypedDict, total=False):
    """An invocation of a tool on an MCP server."""

    id: Required[str]
    """The unique ID of the tool call."""

    arguments: Required[str]
    """A JSON string of the arguments passed to the tool."""

    name: Required[str]
    """The name of the tool that was run."""

    server_label: Required[str]
    """The label of the MCP server running the tool."""

    type: Required[Literal["mcp_call"]]
    """The type of the item. Always `mcp_call`."""

    agent: Optional[McpCallAgent]
    """The agent that produced this item."""

    approval_request_id: Optional[str]
    """
    Unique identifier for the MCP tool call approval request. Include this value in
    a subsequent `mcp_approval_response` input to approve or reject the
    corresponding tool call.
    """

    error: Optional[str]
    """The error from the tool call, if any."""

    output: Optional[str]
    """The output from the tool call."""

    status: Literal["in_progress", "completed", "incomplete", "calling", "failed"]
    """The status of the tool call.

    One of `in_progress`, `completed`, `incomplete`, `calling`, or `failed`.
    """


class CompactionTriggerAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class CompactionTrigger(TypedDict, total=False):
    """Compacts the current context. Must be the final input item."""

    type: Required[Literal["compaction_trigger"]]
    """The type of the item. Always `compaction_trigger`."""

    agent: Optional[CompactionTriggerAgent]
    """The agent that produced this item."""


class ItemReferenceAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ItemReference(TypedDict, total=False):
    """An internal identifier for an item to reference."""

    id: Required[str]
    """The ID of the item to reference."""

    agent: Optional[ItemReferenceAgent]
    """The agent that produced this item."""

    type: Optional[Literal["item_reference"]]
    """The type of item to reference. Always `item_reference`."""


class ProgramAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class Program(TypedDict, total=False):
    id: Required[str]
    """The unique ID of this program item."""

    call_id: Required[str]
    """The stable call ID of the program item."""

    code: Required[str]
    """The JavaScript source executed by programmatic tool calling."""

    fingerprint: Required[str]
    """Opaque program replay fingerprint that must be round-tripped."""

    type: Required[Literal["program"]]
    """The item type. Always `program`."""

    agent: Optional[ProgramAgent]
    """The agent that produced this item."""


class ProgramOutputAgent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class ProgramOutput(TypedDict, total=False):
    id: Required[str]
    """The unique ID of this program output item."""

    call_id: Required[str]
    """The call ID of the program item."""

    result: Required[str]
    """The result produced by the program item."""

    status: Required[Literal["completed", "incomplete"]]
    """The terminal status of the program output."""

    type: Required[Literal["program_output"]]
    """The item type. Always `program_output`."""

    agent: Optional[ProgramOutputAgent]
    """The agent that produced this item."""


BetaResponseInputItemParam: TypeAlias = Union[
    BetaEasyInputMessageParam,
    Message,
    BetaResponseOutputMessageParam,
    BetaResponseFileSearchToolCallParam,
    BetaResponseComputerToolCallParam,
    ComputerCallOutput,
    BetaResponseFunctionWebSearchParam,
    BetaResponseFunctionToolCallParam,
    FunctionCallOutput,
    AgentMessage,
    MultiAgentCall,
    MultiAgentCallOutput,
    ToolSearchCall,
    BetaResponseToolSearchOutputItemParamParam,
    AdditionalTools,
    BetaResponseReasoningItemParam,
    BetaResponseCompactionItemParamParam,
    ImageGenerationCall,
    BetaResponseCodeInterpreterToolCallParam,
    LocalShellCall,
    LocalShellCallOutput,
    ShellCall,
    ShellCallOutput,
    ApplyPatchCall,
    ApplyPatchCallOutput,
    McpListTools,
    McpApprovalRequest,
    McpApprovalResponse,
    McpCall,
    BetaResponseCustomToolCallOutputParam,
    BetaResponseCustomToolCallParam,
    CompactionTrigger,
    ItemReference,
    Program,
    ProgramOutput,
]

BetaResponseInputParam: TypeAlias = List[BetaResponseInputItemParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...... import _legacy_response
from .api_keys import (
    APIKeys,
    AsyncAPIKeys,
    APIKeysWithRawResponse,
    AsyncAPIKeysWithRawResponse,
    APIKeysWithStreamingResponse,
    AsyncAPIKeysWithStreamingResponse,
)
from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ......pagination import SyncConversationCursorPage, AsyncConversationCursorPage
from ......_base_client import AsyncPaginator, make_request_options
from ......types.admin.organization.projects import (
    service_account_list_params,
    service_account_create_params,
    service_account_update_params,
)
from ......types.admin.organization.projects.project_service_account import ProjectServiceAccount
from ......types.admin.organization.projects.service_account_create_response import ServiceAccountCreateResponse
from ......types.admin.organization.projects.service_account_delete_response import ServiceAccountDeleteResponse

__all__ = ["ServiceAccounts", "AsyncServiceAccounts"]


class ServiceAccounts(SyncAPIResource):
    @cached_property
    def api_keys(self) -> APIKeys:
        return APIKeys(self._client)

    @cached_property
    def with_raw_response(self) -> ServiceAccountsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return ServiceAccountsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ServiceAccountsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return ServiceAccountsWithStreamingResponse(self)

    def create(
        self,
        project_id: str,
        *,
        name: str,
        create_service_account_only: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceAccountCreateResponse:
        """Creates a new service account in the project.

        By default, this also returns an
        unredacted API key for the service account.

        Args:
          name: The name of the service account being created.

          create_service_account_only: Create the service account without default roles or an API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._post(
            path_template("/organization/projects/{project_id}/service_accounts", project_id=project_id),
            body=maybe_transform(
                {
                    "name": name,
                    "create_service_account_only": create_service_account_only,
                },
                service_account_create_params.ServiceAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ServiceAccountCreateResponse,
        )

    def retrieve(
        self,
        service_account_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectServiceAccount:
        """
        Retrieves a service account in the project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not service_account_id:
            raise ValueError(f"Expected a non-empty value for `service_account_id` but received {service_account_id!r}")
        return self._get(
            path_template(
                "/organization/projects/{project_id}/service_accounts/{service_account_id}",
                project_id=project_id,
                service_account_id=service_account_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectServiceAccount,
        )

    def update(
        self,
        service_account_id: str,
        *,
        project_id: str,
        name: str | Omit = omit,
        role: Literal["member", "owner"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectServiceAccount:
        """
        Updates a service account in the project.

        Args:
          name: The updated service account name.

          role: The updated service account role.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not service_account_id:
            raise ValueError(f"Expected a non-empty value for `service_account_id` but received {service_account_id!r}")
        return self._post(
            path_template(
                "/organization/projects/{project_id}/service_accounts/{service_account_id}",
                project_id=project_id,
                service_account_id=service_account_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "role": role,
                },
                service_account_update_params.ServiceAccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectServiceAccount,
        )

    def list(
        self,
        project_id: str,
        *,
        after: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncConversationCursorPage[ProjectServiceAccount]:
        """
        Returns a list of service accounts in the project.

        Args:
          after: A cursor for use in pagination. `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._get_api_list(
            path_template("/organization/projects/{project_id}/service_accounts", project_id=project_id),
            page=SyncConversationCursorPage[ProjectServiceAccount],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                    },
                    service_account_list_params.ServiceAccountListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=ProjectServiceAccount,
        )

    def delete(
        self,
        service_account_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceAccountDeleteResponse:
        """
        Deletes a service account from the project.

        Returns confirmation of service account deletion, or an error if the project is
        archived (archived projects have no service accounts).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not service_account_id:
            raise ValueError(f"Expected a non-empty value for `service_account_id` but received {service_account_id!r}")
        return self._delete(
            path_template(
                "/organization/projects/{project_id}/service_accounts/{service_account_id}",
                project_id=project_id,
                service_account_id=service_account_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ServiceAccountDeleteResponse,
        )


class AsyncServiceAccounts(AsyncAPIResource):
    @cached_property
    def api_keys(self) -> AsyncAPIKeys:
        return AsyncAPIKeys(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncServiceAccountsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncServiceAccountsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncServiceAccountsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncServiceAccountsWithStreamingResponse(self)

    async def create(
        self,
        project_id: str,
        *,
        name: str,
        create_service_account_only: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceAccountCreateResponse:
        """Creates a new service account in the project.

        By default, this also returns an
        unredacted API key for the service account.

        Args:
          name: The name of the service account being created.

          create_service_account_only: Create the service account without default roles or an API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._post(
            path_template("/organization/projects/{project_id}/service_accounts", project_id=project_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "create_service_account_only": create_service_account_only,
                },
                service_account_create_params.ServiceAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ServiceAccountCreateResponse,
        )

    async def retrieve(
        self,
        service_account_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectServiceAccount:
        """
        Retrieves a service account in the project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not service_account_id:
            raise ValueError(f"Expected a non-empty value for `service_account_id` but received {service_account_id!r}")
        return await self._get(
            path_template(
                "/organization/projects/{project_id}/service_accounts/{service_account_id}",
                project_id=project_id,
                service_account_id=service_account_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectServiceAccount,
        )

    async def update(
        self,
        service_account_id: str,
        *,
        project_id: str,
        name: str | Omit = omit,
        role: Literal["member", "owner"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectServiceAccount:
        """
        Updates a service account in the project.

        Args:
          name: The updated service account name.

          role: The updated service account role.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not service_account_id:
            raise ValueError(f"Expected a non-empty value for `service_account_id` but received {service_account_id!r}")
        return await self._post(
            path_template(
                "/organization/projects/{project_id}/service_accounts/{service_account_id}",
                project_id=project_id,
                service_account_id=service_account_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "role": role,
                },
                service_account_update_params.ServiceAccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectServiceAccount,
        )

    def list(
        self,
        project_id: str,
        *,
        after: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ProjectServiceAccount, AsyncConversationCursorPage[ProjectServiceAccount]]:
        """
        Returns a list of service accounts in the project.

        Args:
          after: A cursor for use in pagination. `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._get_api_list(
            path_template("/organization/projects/{project_id}/service_accounts", project_id=project_id),
            page=AsyncConversationCursorPage[ProjectServiceAccount],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                    },
                    service_account_list_params.ServiceAccountListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=ProjectServiceAccount,
        )

    async def delete(
        self,
        service_account_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceAccountDeleteResponse:
        """
        Deletes a service account from the project.

        Returns confirmation of service account deletion, or an error if the project is
        archived (archived projects have no service accounts).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not service_account_id:
            raise ValueError(f"Expected a non-empty value for `service_account_id` but received {service_account_id!r}")
        return await self._delete(
            path_template(
                "/organization/projects/{project_id}/service_accounts/{service_account_id}",
                project_id=project_id,
                service_account_id=service_account_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ServiceAccountDeleteResponse,
        )


class ServiceAccountsWithRawResponse:
    def __init__(self, service_accounts: ServiceAccounts) -> None:
        self._service_accounts = service_accounts

        self.create = _legacy_response.to_raw_response_wrapper(
            service_accounts.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            service_accounts.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            service_accounts.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            service_accounts.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            service_accounts.delete,
        )

    @cached_property
    def api_keys(self) -> APIKeysWithRawResponse:
        return APIKeysWithRawResponse(self._service_accounts.api_keys)


class AsyncServiceAccountsWithRawResponse:
    def __init__(self, service_accounts: AsyncServiceAccounts) -> None:
        self._service_accounts = service_accounts

        self.create = _legacy_response.async_to_raw_response_wrapper(
            service_accounts.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            service_accounts.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            service_accounts.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            service_accounts.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            service_accounts.delete,
        )

    @cached_property
    def api_keys(self) -> AsyncAPIKeysWithRawResponse:
        return AsyncAPIKeysWithRawResponse(self._service_accounts.api_keys)


class ServiceAccountsWithStreamingResponse:
    def __init__(self, service_accounts: ServiceAccounts) -> None:
        self._service_accounts = service_accounts

        self.create = to_streamed_response_wrapper(
            service_accounts.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            service_accounts.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            service_accounts.update,
        )
        self.list = to_streamed_response_wrapper(
            service_accounts.list,
        )
        self.delete = to_streamed_response_wrapper(
            service_accounts.delete,
        )

    @cached_property
    def api_keys(self) -> APIKeysWithStreamingResponse:
        return APIKeysWithStreamingResponse(self._service_accounts.api_keys)


class AsyncServiceAccountsWithStreamingResponse:
    def __init__(self, service_accounts: AsyncServiceAccounts) -> None:
        self._service_accounts = service_accounts

        self.create = async_to_streamed_response_wrapper(
            service_accounts.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            service_accounts.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            service_accounts.update,
        )
        self.list = async_to_streamed_response_wrapper(
            service_accounts.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            service_accounts.delete,
        )

    @cached_property
    def api_keys(self) -> AsyncAPIKeysWithStreamingResponse:
        return AsyncAPIKeysWithStreamingResponse(self._service_accounts.api_keys)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BetaResponseInputTextContentParam", "PromptCacheBreakpoint"]


class PromptCacheBreakpoint(TypedDict, total=False):
    """Marks the exact end of a reusable prompt prefix.

    The breakpoint inherits its TTL from the request's `prompt_cache_options.ttl`; the boundary is not rounded to a token block.
    """

    mode: Required[Literal["explicit"]]
    """The breakpoint mode. Always `explicit`."""


class BetaResponseInputTextContentParam(TypedDict, total=False):
    """A text input to the model."""

    text: Required[str]
    """The text input to the model."""

    type: Required[Literal["input_text"]]
    """The type of the input item. Always `input_text`."""

    prompt_cache_breakpoint: Optional[PromptCacheBreakpoint]
    """Marks the exact end of a reusable prompt prefix.

    The breakpoint inherits its TTL from the request's `prompt_cache_options.ttl`;
    the boundary is not rounded to a token block.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from .response_input_file_content import ResponseInputFileContent
from .response_input_text_content import ResponseInputTextContent
from .response_input_image_content import ResponseInputImageContent

__all__ = ["ResponseFunctionCallOutputItem"]

ResponseFunctionCallOutputItem: TypeAlias = Annotated[
    Union[ResponseInputTextContent, ResponseInputImageContent, ResponseInputFileContent],
    PropertyInfo(discriminator="type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import threading
from typing import Any, Callable

EventHandler = Callable[..., Any]


class EventHandlerRegistry:
    """Thread-safe (optional) registry of event handlers."""

    def __init__(self, *, use_lock: bool = False) -> None:
        self._handlers: dict[str, list[EventHandler]] = {}
        self._once_ids: set[int] = set()
        self._lock: threading.Lock | None = threading.Lock() if use_lock else None

    def _acquire(self) -> None:
        if self._lock is not None:
            self._lock.acquire()

    def _release(self) -> None:
        if self._lock is not None:
            self._lock.release()

    def add(self, event_type: str, handler: EventHandler, *, once: bool = False) -> None:
        self._acquire()
        try:
            handlers = self._handlers.setdefault(event_type, [])
            handlers.append(handler)
            if once:
                self._once_ids.add(id(handler))
        finally:
            self._release()

    def remove(self, event_type: str, handler: EventHandler) -> None:
        self._acquire()
        try:
            handlers = self._handlers.get(event_type)
            if handlers is not None:
                try:
                    handlers.remove(handler)
                except ValueError:
                    pass
                self._once_ids.discard(id(handler))
        finally:
            self._release()

    def get_handlers(self, event_type: str) -> list[EventHandler]:
        """Return a snapshot of handlers for the given event type, removing once-handlers."""
        self._acquire()
        try:
            handlers = self._handlers.get(event_type)
            if not handlers:
                return []
            result = list(handlers)
            to_remove = [h for h in result if id(h) in self._once_ids]
            for h in to_remove:
                handlers.remove(h)
                self._once_ids.discard(id(h))
            return result
        finally:
            self._release()

    def has_handlers(self, event_type: str) -> bool:
        self._acquire()
        try:
            handlers = self._handlers.get(event_type)
            return bool(handlers)
        finally:
            self._release()

    def merge_into(self, target: EventHandlerRegistry) -> None:
        """Move all handlers from this registry into *target*, then clear self."""
        self._acquire()
        try:
            for event_type, handlers in self._handlers.items():
                for handler in handlers:
                    once = id(handler) in self._once_ids
                    target.add(event_type, handler, once=once)
            self._handlers.clear()
            self._once_ids.clear()
        finally:
            self._release()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ServiceAccountCreateResponse", "APIKey"]


class APIKey(BaseModel):
    id: str

    created_at: int

    name: str

    object: Literal["organization.project.service_account.api_key"]
    """The object type, which is always `organization.project.service_account.api_key`"""

    value: str


class ServiceAccountCreateResponse(BaseModel):
    id: str

    api_key: Optional[APIKey] = None

    created_at: int

    name: str

    object: Literal["organization.project.service_account"]

    role: Literal["member", "none"]
    """Service accounts created with default project membership have role `member`.

    Accounts created with `create_service_account_only` have role `none`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ...._utils import PropertyInfo
from ...._models import BaseModel
from .chatkit_attachment import ChatKitAttachment

__all__ = [
    "ChatKitThreadUserMessageItem",
    "Content",
    "ContentInputText",
    "ContentQuotedText",
    "InferenceOptions",
    "InferenceOptionsToolChoice",
]


class ContentInputText(BaseModel):
    """Text block that a user contributed to the thread."""

    text: str
    """Plain-text content supplied by the user."""

    type: Literal["input_text"]
    """Type discriminator that is always `input_text`."""


class ContentQuotedText(BaseModel):
    """Quoted snippet that the user referenced in their message."""

    text: str
    """Quoted text content."""

    type: Literal["quoted_text"]
    """Type discriminator that is always `quoted_text`."""


Content: TypeAlias = Annotated[Union[ContentInputText, ContentQuotedText], PropertyInfo(discriminator="type")]


class InferenceOptionsToolChoice(BaseModel):
    """Preferred tool to invoke. Defaults to null when ChatKit should auto-select."""

    id: str
    """Identifier of the requested tool."""


class InferenceOptions(BaseModel):
    """Inference overrides applied to the message. Defaults to null when unset."""

    model: Optional[str] = None
    """Model name that generated the response.

    Defaults to null when using the session default.
    """

    tool_choice: Optional[InferenceOptionsToolChoice] = None
    """Preferred tool to invoke. Defaults to null when ChatKit should auto-select."""


class ChatKitThreadUserMessageItem(BaseModel):
    """User-authored messages within a thread."""

    id: str
    """Identifier of the thread item."""

    attachments: List[ChatKitAttachment]
    """Attachments associated with the user message. Defaults to an empty list."""

    content: List[Content]
    """Ordered content elements supplied by the user."""

    created_at: int
    """Unix timestamp (in seconds) for when the item was created."""

    inference_options: Optional[InferenceOptions] = None
    """Inference overrides applied to the message. Defaults to null when unset."""

    object: Literal["chatkit.thread_item"]
    """Type discriminator that is always `chatkit.thread_item`."""

    thread_id: str
    """Identifier of the parent thread."""

    type: Literal["chatkit.user_message"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.types.admin.organization import (
    OrganizationSpendLimit,
    OrganizationSpendLimitDeleted,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSpendLimit:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: OpenAI) -> None:
        spend_limit = client.admin.organization.spend_limit.retrieve()
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: OpenAI) -> None:
        response = client.admin.organization.spend_limit.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend_limit = response.parse()
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: OpenAI) -> None:
        with client.admin.organization.spend_limit.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend_limit = response.parse()
            assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_update(self, client: OpenAI) -> None:
        spend_limit = client.admin.organization.spend_limit.update(
            currency="USD",
            interval="month",
            threshold_amount=1,
        )
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: OpenAI) -> None:
        response = client.admin.organization.spend_limit.with_raw_response.update(
            currency="USD",
            interval="month",
            threshold_amount=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend_limit = response.parse()
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: OpenAI) -> None:
        with client.admin.organization.spend_limit.with_streaming_response.update(
            currency="USD",
            interval="month",
            threshold_amount=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend_limit = response.parse()
            assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: OpenAI) -> None:
        spend_limit = client.admin.organization.spend_limit.delete()
        assert_matches_type(OrganizationSpendLimitDeleted, spend_limit, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: OpenAI) -> None:
        response = client.admin.organization.spend_limit.with_raw_response.delete()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend_limit = response.parse()
        assert_matches_type(OrganizationSpendLimitDeleted, spend_limit, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: OpenAI) -> None:
        with client.admin.organization.spend_limit.with_streaming_response.delete() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend_limit = response.parse()
            assert_matches_type(OrganizationSpendLimitDeleted, spend_limit, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSpendLimit:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOpenAI) -> None:
        spend_limit = await async_client.admin.organization.spend_limit.retrieve()
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.spend_limit.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend_limit = response.parse()
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.spend_limit.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend_limit = await response.parse()
            assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_update(self, async_client: AsyncOpenAI) -> None:
        spend_limit = await async_client.admin.organization.spend_limit.update(
            currency="USD",
            interval="month",
            threshold_amount=1,
        )
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.spend_limit.with_raw_response.update(
            currency="USD",
            interval="month",
            threshold_amount=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend_limit = response.parse()
        assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.spend_limit.with_streaming_response.update(
            currency="USD",
            interval="month",
            threshold_amount=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend_limit = await response.parse()
            assert_matches_type(OrganizationSpendLimit, spend_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncOpenAI) -> None:
        spend_limit = await async_client.admin.organization.spend_limit.delete()
        assert_matches_type(OrganizationSpendLimitDeleted, spend_limit, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.spend_limit.with_raw_response.delete()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend_limit = response.parse()
        assert_matches_type(OrganizationSpendLimitDeleted, spend_limit, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.spend_limit.with_streaming_response.delete() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend_limit = await response.parse()
            assert_matches_type(OrganizationSpendLimitDeleted, spend_limit, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["OrganizationSpendAlert", "NotificationChannel"]


class NotificationChannel(BaseModel):
    """Email notification settings for a spend alert."""

    recipients: List[str]
    """Email addresses that receive the spend alert notification."""

    type: Literal["email"]
    """The notification channel type. Currently only `email` is supported."""

    subject_prefix: Optional[str] = None
    """Optional subject prefix for alert emails."""


class OrganizationSpendAlert(BaseModel):
    """Represents a spend alert configured at the organization level."""

    id: str
    """The identifier, which can be referenced in API endpoints."""

    currency: Literal["USD"]
    """The currency for the threshold amount."""

    interval: Literal["month"]
    """The time interval for evaluating spend against the threshold."""

    notification_channel: NotificationChannel
    """Email notification settings for a spend alert."""

    object: Literal["organization.spend_alert"]
    """The object type, which is always `organization.spend_alert`."""

    threshold_amount: int
    """The alert threshold amount, in cents."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from .._types import FileTypes
from .video_seconds import VideoSeconds

__all__ = ["VideoExtendParams", "Video", "VideoVideoReferenceInputParam"]


class VideoExtendParams(TypedDict, total=False):
    prompt: Required[str]
    """Updated text prompt that directs the extension generation."""

    seconds: Required[VideoSeconds]
    """
    Length of the newly generated extension segment in seconds (allowed values: 4,
    8, 12, 16, 20).
    """

    video: Required[Video]
    """Reference to the completed video to extend."""


class VideoVideoReferenceInputParam(TypedDict, total=False):
    """Reference to the completed video."""

    id: Required[str]
    """The identifier of the completed video."""


Video: TypeAlias = Union[FileTypes, VideoVideoReferenceInputParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RateLimitsUpdatedEvent", "RateLimit"]


class RateLimit(BaseModel):
    limit: Optional[int] = None
    """The maximum allowed value for the rate limit."""

    name: Optional[Literal["requests", "tokens"]] = None
    """The name of the rate limit (`requests`, `tokens`)."""

    remaining: Optional[int] = None
    """The remaining value before the limit is reached."""

    reset_seconds: Optional[float] = None
    """Seconds until the rate limit resets."""


class RateLimitsUpdatedEvent(BaseModel):
    """Emitted at the beginning of a Response to indicate the updated rate limits.


    When a Response is created some tokens will be "reserved" for the output
    tokens, the rate limits shown here reflect that reservation, which is then
    adjusted accordingly once the Response is completed.
    """

    event_id: str
    """The unique ID of the server event."""

    rate_limits: List[RateLimit]
    """List of rate limit information."""

    type: Literal["rate_limits.updated"]
    """The event type, must be `rate_limits.updated`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BetaResponseFileSearchCallCompletedEvent", "Agent"]


class Agent(BaseModel):
    """The agent that owns this multi-agent streaming event."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseFileSearchCallCompletedEvent(BaseModel):
    """Emitted when a file search call is completed (results found)."""

    item_id: str
    """The ID of the output item that the file search call is initiated."""

    output_index: int
    """The index of the output item that the file search call is initiated."""

    sequence_number: int
    """The sequence number of this event."""

    type: Literal["response.file_search_call.completed"]
    """The type of the event. Always `response.file_search_call.completed`."""

    agent: Optional[Agent] = None
    """The agent that owns this multi-agent streaming event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import pytest

from openai._exceptions import WebSocketQueueFullError
from openai._send_queue import SendQueue


class TestSendQueue:
    def test_enqueue_and_drain(self) -> None:
        q = SendQueue()
        q.enqueue('{"type": "session.update"}')
        q.enqueue('{"type": "response.create"}')
        assert len(q) == 2

        items = q.drain()
        assert items == ['{"type": "session.update"}', '{"type": "response.create"}']
        assert len(q) == 0

    def test_enqueue_respects_byte_limit(self) -> None:
        q = SendQueue(max_bytes=10)
        q.enqueue("12345")  # 5 bytes, fits
        with pytest.raises(WebSocketQueueFullError):
            q.enqueue("123456")  # 6 bytes, would exceed 10
        assert len(q) == 1

    def test_drain_empties_queue(self) -> None:
        q = SendQueue()
        q.enqueue("hello")
        q.drain()
        assert len(q) == 0
        assert not q

    def test_bool(self) -> None:
        q = SendQueue()
        assert not q
        q.enqueue("x")
        assert q

    def test_flush_sync(self) -> None:
        q = SendQueue()
        q.enqueue("a")
        q.enqueue("b")
        q.enqueue("c")

        sent: list[str] = []
        q.flush_sync(sent.append)
        assert sent == ["a", "b", "c"]
        assert len(q) == 0

    def test_flush_sync_requeues_on_failure(self) -> None:
        q = SendQueue()
        q.enqueue("a")
        q.enqueue("b")
        q.enqueue("c")

        sent: list[str] = []

        def failing_send(data: str) -> None:
            if data == "b":
                raise RuntimeError("send failed")
            sent.append(data)

        with pytest.raises(RuntimeError, match="send failed"):
            q.flush_sync(failing_send)

        assert sent == ["a"]
        # b and c should be re-queued
        remaining = q.drain()
        assert remaining == ["b", "c"]

    @pytest.mark.asyncio
    async def test_flush_async(self) -> None:
        q = SendQueue()
        q.enqueue("a")
        q.enqueue("b")

        sent: list[str] = []

        async def async_send(data: str) -> None:
            sent.append(data)

        await q.flush_async(async_send)
        assert sent == ["a", "b"]
        assert len(q) == 0

    @pytest.mark.asyncio
    async def test_flush_async_requeues_on_failure(self) -> None:
        q = SendQueue()
        q.enqueue("a")
        q.enqueue("b")
        q.enqueue("c")

        sent: list[str] = []

        async def failing_send(data: str) -> None:
            if data == "b":
                raise RuntimeError("send failed")
            sent.append(data)

        with pytest.raises(RuntimeError, match="send failed"):
            await q.flush_async(failing_send)

        assert sent == ["a"]
        remaining = q.drain()
        assert remaining == ["b", "c"]

    def test_flush_sync_preserves_new_items_on_failure(self) -> None:
        """If items are enqueued after flush starts and flush fails,
        the re-queued items should come before the new items."""
        q = SendQueue()
        q.enqueue("a")
        q.enqueue("b")

        def failing_send(data: str) -> None:
            if data == "b":
                # Simulate another thread enqueuing during flush
                q.enqueue("new")
                raise RuntimeError("fail")

        with pytest.raises(RuntimeError):
            q.flush_sync(failing_send)

        # "b" (failed) should come before "new" (added during flush)
        remaining = q.drain()
        assert remaining == ["b", "new"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .skill_version import SkillVersion as SkillVersion
from .skill_version_list import SkillVersionList as SkillVersionList
from .version_list_params import VersionListParams as VersionListParams
from .deleted_skill_version import DeletedSkillVersion as DeletedSkillVersion
from .version_create_params import VersionCreateParams as VersionCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["OrganizationSpendLimitDeleted"]


class OrganizationSpendLimitDeleted(BaseModel):
    """Confirmation payload returned after deleting an organization hard spend limit."""

    deleted: bool
    """Whether the hard spend limit was deleted."""

    object: Literal["organization.spend_limit.deleted"]
    """The object type, which is always `organization.spend_limit.deleted`."""
